require_relative '../../lib/kiss'
require 'abstractify'

ENCODED_FRAME = [192, 0, 158, 154, 142, 64, 64, 64, 96, 174, 100, 142, 154, 136, 64, 98, 174, 146, 136, 138, 98, 64, 98, 174, 146, 136, 138, 100, 64, 101, 3, 240, 116, 101, 115, 116, 95, 101, 110, 99, 111, 100, 101, 95, 102, 114, 97, 109, 101, 192]

DECODED_FRAME = {
                  :source => " W2GMD-1",
                  :destination => "OMG",
                  :payload => "test_encode_frame",
                  :path => ['WIDE1-1', 'WIDE2-2']
                }

class KissMock < Kiss::KissAbstract

    def initialize(strip_df_start=true)
        super(strip_df_start)
        @read_from_interface = []
        @sent_to_interface = []
    end

    public
    def connect(mode_init=nil, *args, **kwargs)
    end

    public
    def close(*args, **kwargs)
    end

    protected
    def read_interface
        if @read_from_interface.length == 0
            return nil
        end
        return @read_from_interface.shift
    end

    protected
    def write_interface(data)
        @sent_to_interface << data
    end

    public
    def clear_interface
        @read_from_interface = []
        @sent_to_interface = []
    end

    public
    def add_read_from_interface(raw_frame)
        @read_from_interface << raw_frame
    end

    public
    def get_sent_to_interface
        return @sent_to_interface
    end
end

describe Kiss::KissAbstract do

  describe ".new" do
    context "Given a concrete child class" do
      it "is successfully instantiated" do
        kiss_mock = KissMock.new
        expect(kiss_mock).to_not be_nil
      end
    end
    context "Given the abstract class itself" do
      it "will throw an AbstractClassError" do
        expect {
          Kiss::KissAbstract.new
        }.to raise_error(Abstractify::AbstractClassError)
      end
    end
  end

  describe ".read" do
    context "Given an encoded frame as bytes on the underlying interface" do
      let(:kiss_mock) {KissMock.new}
      it "successfully decoded and parsed the frame" do
        kiss_mock.add_read_from_interface(ENCODED_FRAME)
        translated_frame = kiss_mock.read
        expect(translated_frame).to eql(DECODED_FRAME)
      end
    end

    context "Given no bytes on the underlying interface" do
      let(:kiss_mock) {KissMock.new}
      it "successfully decoded and parsed the frame" do
        translated_frame = kiss_mock.read
        expect(translated_frame).to be_nil
      end
    end
  end

  describe ".write" do
    context "Given a frame as a map of strings" do
      let(:kiss_mock) {KissMock.new}
      it "successfully encodes to the underlying interface" do
        kiss_mock.write(DECODED_FRAME)
        all_raw_frames = kiss_mock.get_sent_to_interface
        expect(all_raw_frames[0]).to eql(ENCODED_FRAME)
      end
    end
    context "Given a frame that encodes but is invalid" do
      let(:kiss_mock) {KissMock.new}
      it "successfully encodes to the underlying interface" do
        allow(Kiss::KissAbstract).to receive(:valid_frame).and_return(false)

        expect {
          kiss_mock.write(DECODED_FRAME)
        }.to raise_error(IOError)
      end
    end
  end

end
